@@ -66,7 +66,7 @@
 #define ZIP(x)	 ZIPJAZ(x,96, 64, 32, 0)
 #define RZIP(x)	 ZIPJAZ(x,96, 64, 32, SCSI_FLAG|PRIV_FLAG)
 
-#define REMOTE    {"$DISPLAY", 'X', 0,0, 0,0, 0,0,0L, DEF_ARG0(FLOPPYD_FLAG)}
+#define REMOTE    {"$DISPLAY", 'X', 0,0, 0,0, 0,0,0L, DEF_ARG0(FLOPPYD_FLAG),0,0}
 
 
 
@@ -730,12 +730,12 @@ int analyze_one_reply(RawRequest_t *raw_cmd, int *bytes, int do_print)
 
 #define predefined_devices
 struct device devices[] = {
-	{"/dev/fd0", 'A', 0, 0, 80,2, 18,0, MDEF_ARG},
-	{"/dev/fd1", 'B', 0, 0, 0,0, 0,0, FDEF_ARG},
+	{"/dev/fd0", 'A', 0, 0, 80,2, 18,0, MDEF_ARG, 0, 0},
+	{"/dev/fd1", 'B', 0, 0, 0,0, 0,0, FDEF_ARG, 0, 0},
 	/* we assume that the Zip or Jaz drive is the second on the SCSI bus */
-	{"/dev/sdb4",'J', GENHD },
-	{"/dev/sdb4",'Z', GENHD },
-	/*	{"/dev/sda4",'D', GENHD },*/
+	{"/dev/sdb4",'J', GENHD, 0, 0 },
+	{"/dev/sdb4",'Z', GENHD, 0, 0 },
+	/*	{"/dev/sda4",'D', GENHD, 0, 0 },*/
 	REMOTE
 };
 
@@ -955,7 +955,9 @@ struct device devices[] = {
 
 #ifdef OS_mingw32msvc
 #define predefined_devices
-struct device devices[] = {};
+struct device devices[] = {
+   {"\\\\.\\A:", 'A', GENFD },
+};
 #endif
 
 #ifdef INIT_GENERIC
