@@ -53,13 +53,6 @@ static __inline__ int forceReadSector(Fs_t *This, char *buf, unsigned int off,
 }
 
 
-static __inline__ int writeSector(Fs_t *This, char *buf, unsigned int off,
-				  size_t size)
-{
-	return WRITES(This->Next, buf, sectorsToBytes((Stream_t*)This, off), 
-				  size << This->sectorShift);
-}
-
 static __inline__ int forceWriteSector(Fs_t *This, char *buf, unsigned int off,
 				       size_t size)
 {
@@ -415,7 +408,6 @@ void fat_write(Fs_t *This)
 {
 	unsigned int i, j, dups, bit, slot;
 	int ret;
-	int fat_start;
 
 	/*fprintf(stderr, "Fat write\n");*/
 
@@ -429,7 +421,6 @@ void fat_write(Fs_t *This)
 
 	for(i=0; i<dups; i++){
 		j = 0;
-		fat_start = This->fat_start + i*This->fat_len;
 		for(slot=0;j<This->fat_len;slot++) {
 			if(!This->FatMap[slot].dirty) {
 				j += SECT_PER_ENTRY;
@@ -468,7 +459,7 @@ void fat_write(Fs_t *This)
 		set_dword(infoSector->signature2, INFOSECT_SIGNATURE2);
 		set_dword(infoSector->pos, This->last);
 		set_dword(infoSector->count, This->freeSpace);
-		set_dword(infoSector->signature3, 0xaa55);
+		set_word(infoSector->signature3, 0xaa55);
 		if(forceWriteSector(This, (char *)infoSector, This->infoSectorLoc, 1) !=
 		   (signed int) This->sector_size)
 			fprintf(stderr,"Trouble writing the info sector\n");
@@ -625,7 +616,7 @@ static int check_fat(Fs_t *This)
  * Read the first sector of FAT table into memory.  Crude error detection on
  * wrong FAT encoding scheme.
  */
-static int check_media_type(Fs_t *This, struct bootsector *boot, 
+static int check_media_type(Fs_t *This, union bootsector *boot, 
 			    unsigned int tot_sectors)
 {
 	unsigned char *address;
@@ -653,13 +644,13 @@ static int check_media_type(Fs_t *This, struct bootsector *boot,
 		 * and 0xff.  Do not consider this as an error */
 		return 0;
 	
-	if((address[0] != boot->descr && boot->descr >= 0xf0 &&
+	if((address[0] != boot->boot.descr && boot->boot.descr >= 0xf0 &&
 	    ((address[0] != 0xf9 && address[0] != 0xf7) 
-	     || boot->descr != 0xf0)) || address[0] < 0xf0) {
+	     || boot->boot.descr != 0xf0)) || address[0] < 0xf0) {
 		fprintf(stderr,
 			"Bad media types %02x/%02x, probably non-MSDOS disk\n", 
 				address[0],
-				boot->descr);
+				boot->boot.descr);
 		return -1;
 	}
 
@@ -671,14 +662,14 @@ static int check_media_type(Fs_t *This, struct bootsector *boot,
 	return 0;
 }
 
-static int fat_32_read(Fs_t *This, struct bootsector *boot, 
+static int fat_32_read(Fs_t *This, union bootsector *boot, 
 		       unsigned int tot_sectors)
 {
 	int size;
 
 	This->fat_len = DWORD(ext.fat32.bigFat);
-	This->writeAllFats = !(boot->ext.fat32.extFlags[0] & 0x80);
-	This->primaryFat = boot->ext.fat32.extFlags[0] & 0xf;
+	This->writeAllFats = !(boot->boot.ext.fat32.extFlags[0] & 0x80);
+	This->primaryFat = boot->boot.ext.fat32.extFlags[0] & 0xf;
 	This->rootCluster = DWORD(ext.fat32.rootCluster);
 	This->clus_start = This->fat_start + This->num_fat * This->fat_len;
 
@@ -701,14 +692,13 @@ static int fat_32_read(Fs_t *This, struct bootsector *boot,
 	}
 	
 	set_fat32(This);
-	return(check_media_type(This,boot, tot_sectors) ||
+	return(check_media_type(This, boot, tot_sectors) ||
 	       check_fat(This));
 }
 
 
-static int old_fat_read(Fs_t *This, struct bootsector *boot, 
-						int config_fat_bits,
-						size_t tot_sectors, int nodups)
+static int old_fat_read(Fs_t *This, union bootsector *boot, 
+			size_t tot_sectors, int nodups)
 {
 	This->writeAllFats = 1;
 	This->primaryFat = 0;
@@ -737,7 +727,7 @@ static int old_fat_read(Fs_t *This, struct bootsector *boot,
  * Read the first sector of the  FAT table into memory and initialize 
  * structures.
  */
-int fat_read(Fs_t *This, struct bootsector *boot, int fat_bits,
+int fat_read(Fs_t *This, union bootsector *boot,
 	   size_t tot_sectors, int nodups)
 {
 	This->fat_error = 0;
@@ -748,7 +738,7 @@ int fat_read(Fs_t *This, struct bootsector *boot, int fat_bits,
 	This->lastFatSectorData = 0;
 
 	if(This->fat_len)
-		return old_fat_read(This, boot, fat_bits, tot_sectors, nodups);
+		return old_fat_read(This, boot, tot_sectors, nodups);
 	else
 		return fat_32_read(This, boot, tot_sectors);
 }
